#!perl -w
use Test::More;
use strict;
use Config;
use File::Temp qw( tempdir );

use DE_EPAGES::Core::API::File qw ( WriteFile CreateDirectory DeleteDirectory );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Config qw(StoreTypesWithCartridge ShopStoreNames ShopStoreTypes SiteStoreNames SiteStoreTypes);

my $CartridgesXml = <<'END_FILE',
<?xml version="1.0" encoding="UTF-8"?>
<epages>
  <DatabaseType Name="ASPoolDB" >
    <Cartridge Package="DE_EPAGES::Core" />
    <Cartridge Package="DE_EPAGES::XML" />
  </DatabaseType>
  <DatabaseType Name="Site">
    <Cartridge Package="DE_EPAGES::Store" />
    <Cartridge Package="DE_EPAGES::ShopConfiguration" />
  </DatabaseType>
  <DatabaseType Name="Store">
    <Cartridge Package="DE_EPAGES::Core" />
    <Cartridge Package="DE_EPAGES::Shop" />
  </DatabaseType>
  <DatabaseType Name="Store2">
    <Cartridge Package="DE_EPAGES::Calendar" />
    <Cartridge Package="DE_EPAGES::Shop" />
  </DatabaseType>
</epages>
END_FILE

my %StoreConf = (
    ASPoolDB => <<'END_FILE',
[Connection]
active=1
datasource=dbi:SQLite:dbname=_EPAGES_CONFIG_/ASPool.db
StoreType=ASPoolDB
END_FILE
    Backup => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
END_FILE
    EbayCategories => <<'END_FILE',
[Connection]
active=1
datasource=dbi:SQLite:dbname=_EPAGES_CONFIG_/EbayCategories.db
END_FILE
    Site => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
StoreType=Site
END_FILE
    MyStore => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
StoreType=Store
END_FILE
    MyStore2 => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
StoreType=Store2
END_FILE
    );

sub Main {

    plan tests => 5;

    my $ConfigDir = tempdir();
    my $DBDir = "$ConfigDir/Database.d";

    CreateDirectory($DBDir, { recursive => 1 });

    local $ENV{EPAGES_CONFIG} = $ConfigDir;
    local $ENV{EPAGES_CARTRIDGES} = $ConfigDir;

    WriteFile( "$DBDir/$_.conf", \$StoreConf{$_} ) foreach keys %StoreConf;
    WriteFile( "$ConfigDir/Cartridges.xml", \$CartridgesXml );

    # test 1: did we found all stores with DE_EPAGES::Shop cartridge?
    my @ExpectedStoreNames = sort qw(MyStore MyStore2);
    my @FoundStoreNames = sort @{ShopStoreNames()};
    ok(@FoundStoreNames ~~ @ExpectedStoreNames, "(@FoundStoreNames) ~~ (@ExpectedStoreNames)");

    # test 2: did we found all stores types with DE_EPAGES::Shop cartridge?
    my @ExpectedStoreTypes = sort qw(Store Store2);
    my @FoundStoreTypes = sort @{ShopStoreTypes()};
    ok(@FoundStoreTypes ~~ @ExpectedStoreTypes, "(@FoundStoreTypes) ~~ (@ExpectedStoreTypes)");

    # test 3: did we found all site stores?
    @ExpectedStoreNames = sort qw(Site);
    @FoundStoreNames = sort @{SiteStoreNames()};
    ok(@FoundStoreNames ~~ @ExpectedStoreNames, "(@FoundStoreNames) ~~ (@ExpectedStoreNames)");

    # test 4: did we found all site stores types?
    @ExpectedStoreTypes = sort qw(Site);
    @FoundStoreTypes = sort @{SiteStoreTypes()};
    ok(@FoundStoreTypes ~~ @ExpectedStoreTypes, "(@FoundStoreTypes) ~~ (@ExpectedStoreTypes)");

    # test 4: did we found all stores types with cartridge "DE_EPAGES::Core"?
    @ExpectedStoreTypes = sort qw(ASPoolDB Store);
    @FoundStoreTypes = sort @{StoreTypesWithCartridge('DE_EPAGES::Core')};
    ok(@FoundStoreTypes ~~ @ExpectedStoreTypes, "(@FoundStoreTypes) ~~ (@ExpectedStoreTypes)");

    DeleteDirectory($ConfigDir, { recursive => 1 });
}

RunScript(Sub=>\&Main);
