use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::MultiStoreMaster::API::AffiliateOverwritable qw ( SetAffiliateOverwritableAttributes GetAffiliateOverwritableAttributes );
use Readonly;
use Test::More;

Readonly my $ROOTPATH => '/Test';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                TestAffiliateOverwritableAttributes();
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub TestAffiliateOverwritableAttributes {
    my $Shop    = LoadObjectByPath('/Test/TestMasterShop');
    is(scalar @{GetAffiliateOverwritableAttributes($Shop, 'Category')}, 0, 'no overwritable attributes');
    SetAffiliateOverwritableAttributes($Shop, 'Category', ['Name', 'Description']);
    is(scalar @{GetAffiliateOverwritableAttributes($Shop, 'Category')}, 2, 'two overwritable attributes');

    SetAffiliateOverwritableAttributes($Shop, 'Category',[]);
    is(scalar @{GetAffiliateOverwritableAttributes($Shop, 'Category')}, 0, 'no overwritable attributes');
    $Shop->set({'OverwritableCategoryAttributes' => ['ListPrice', 'Manufacturer' ]});
    is(scalar @{$Shop->get('OverwritableCategoryAttributes')}, 2, 'using get / set');
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'TestShop.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
